<?php

/**
 * CREATE  TABLE IF NOT EXISTS `dooz`.`call_data` (
 * `id` INT NOT NULL AUTO_INCREMENT ,
 * `callee` VARCHAR(255) NOT NULL ,
 * `data_name` VARCHAR(255) NOT NULL ,
 * `data` BLOB NULL ,
 * `answered` TINYINT(1) NOT NULL ,
 * `action_call_id` INT NOT NULL ,
 * PRIMARY KEY (`id`) ,
 * INDEX `fk_call_data_action_call1` (`action_call_id` ASC) ,
 * CONSTRAINT `fk_call_data_action_call1`
 * FOREIGN KEY (`action_call_id` )
 * REFERENCES `dooz`.`action_call` (`id` )
 * ON DELETE CASCADE
 * ON UPDATE RESTRICT)
 * ENGINE = InnoDB
 *
 * The followings are the available columns in table 'call_data':
 * @property integer $action_call_id
 * @property string $callee
 * @property string $data_name
 * @property string $data
 * @property integer $answered
 *
 * The followings are the available model relations:
 * @property ActionCall $actionCall
 */
class CallData extends CActiveRecord {
    /**
     * @return array
     */
    public function behaviors() {
        $behaviors = parent::behaviors();
        return $behaviors + array(
            'CSerializeBehavior' => array(
                'class' => 'yii.extensions.SerializeBehavior.SerializeBehavior',
                'serialAttributes' => array(    // name of attribute(s)
                    'data',
                ),
            ),
        );
    }
	
	/**
     * Get the first call that you find using this uri for the current session
	 * @param string $uri
	 * @throws CException
	 * @return CallData
	 */
	public static function getCall($uri) {
        $models = self::model()->findAllByAttributes(array(
            'callee' => $uri,
            'answered' => false,
        ));
        foreach($models as $model) {
            if($model->actionCall instanceof ActionCall) {   //this is because of default scope of action call
                return $model;
            }
        }
        return null;
	}

    /**
     * Checks if it is found in a call configuration (2d-array)
     * @param array $calls
     * @return bool
     */
    public function isFoundInCalls($calls) {
        foreach($calls as $call) {
            if( ($this->callee == $call['callee']) && ($this->data_name == $call['data_name']) ) {
                return true;
            }
        }
        return false;
    }
	
	/**
     * We allow for already answered calls to re-answer
	 * @return string
	 */
	public function answerCall() {
        $this->answered = true;

		$transaction = Yii::app()->db->beginTransaction();
		try {
			$this->update(array('answered','data'));

			$transaction->commit();
		}
		catch(CException $e) {
		    $transaction->rollBack();
		}

		return $this->actionCall->caller;
	}

    /**
     * Returns the static model of the specified AR class.
     * @param string $className
     * @return CallData the static model class
     */
	public static function model($className=__CLASS__) {
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName() {
		return '{{call_data}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules() {
		// NOTE: you should only define rules for those attributes that will receive user inputs.
		return array(
			array('action_call_id, callee, data_name, answered', 'required'),
			
			array('action_call_id', 'numerical', 'integerOnly'=>true),

			array('answered', 'in', 'range'=>array(true,false)),
			
			array('callee, data_name', 'length', 'max'=>255),

			array('data', 'safe'),
			// The following rule is used by search(). Please remove those attributes that should not be searched.
			array('callee, data_name, answered', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations() {
		return array(
			'actionCall' => array(self::BELONGS_TO, 'ActionCall', 'action_call_id'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels() {
		return array(
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search() {
		// Warning: Please modify the following code to remove attributes that should not be searched.
		$criteria=new CDbCriteria;
		$criteria->compare('callee',$this->callee,true);
		$criteria->compare('data_name',$this->data_name,true);
		$criteria->compare('answered',$this->answered);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
}